# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"first": "00_export.ipynb",
         "read_nb": "00_export.ipynb",
         "check_re": "00_export.ipynb",
         "check_re_multi": "00_export.ipynb",
         "is_export": "00_export.ipynb",
         "find_default_export": "00_export.ipynb",
         "ReTstFlags": "00_export.ipynb",
         "export_names": "00_export.ipynb",
         "extra_add": "00_export.ipynb",
         "relative_import": "00_export.ipynb",
         "reset_nbdev_module": "00_export.ipynb",
         "get_nbdev_module": "00_export.ipynb",
         "save_nbdev_module": "00_export.ipynb",
         "split_flags_and_code": "00_export.ipynb",
         "create_mod_file": "00_export.ipynb",
         "create_mod_files": "00_export.ipynb",
         "add_init": "00_export.ipynb",
         "update_version": "00_export.ipynb",
         "update_baseurl": "00_export.ipynb",
         "notebook2script": "00_export.ipynb",
         "DocsTestClass": "00_export.ipynb",
         "update_lib_with_exporti_testfn": "00_export.ipynb",
         "get_name": "01_sync.ipynb",
         "qual_name": "01_sync.ipynb",
         "source_nb": "01_sync.ipynb",
         "relimport2name": "01_sync.ipynb",
         "script2notebook": "01_sync.ipynb",
         "diff_nb_script": "01_sync.ipynb",
         "is_enum": "02_showdoc.ipynb",
         "is_lib_module": "02_showdoc.ipynb",
         "try_external_doc_link": "02_showdoc.ipynb",
         "doc_link": "02_showdoc.ipynb",
         "add_doc_links": "02_showdoc.ipynb",
         "get_source_link": "02_showdoc.ipynb",
         "get_nb_source_link": "02_showdoc.ipynb",
         "nb_source_link": "02_showdoc.ipynb",
         "type_repr": "02_showdoc.ipynb",
         "format_param": "02_showdoc.ipynb",
         "show_doc": "02_showdoc.ipynb",
         "parse_nbdev_show_doc": "02_showdoc.ipynb",
         "nbdev_show_doc": "02_showdoc.ipynb",
         "md2html": "02_showdoc.ipynb",
         "get_doc_link": "02_showdoc.ipynb",
         "doc": "02_showdoc.ipynb",
         "HTMLParseAttrs": "03_export2html.ipynb",
         "remove_widget_state": "03_export2html.ipynb",
         "upd_metadata": "03_export2html.ipynb",
         "hide_cells": "03_export2html.ipynb",
         "clean_exports": "03_export2html.ipynb",
         "treat_backticks": "03_export2html.ipynb",
         "add_jekyll_notes": "03_export2html.ipynb",
         "copy_images": "03_export2html.ipynb",
         "adapt_img_path": "03_export2html.ipynb",
         "escape_latex": "03_export2html.ipynb",
         "collapse_cells": "03_export2html.ipynb",
         "remove_hidden": "03_export2html.ipynb",
         "find_default_level": "03_export2html.ipynb",
         "add_show_docs": "03_export2html.ipynb",
         "remove_fake_headers": "03_export2html.ipynb",
         "remove_empty": "03_export2html.ipynb",
         "get_metadata": "03_export2html.ipynb",
         "ExecuteShowDocPreprocessor": "03_export2html.ipynb",
         "execute_nb": "03_export2html.ipynb",
         "cite2link": "03_export2html.ipynb",
         "write_tmpl": "03_export2html.ipynb",
         "write_tmpls": "03_export2html.ipynb",
         "nbdev_exporter": "03_export2html.ipynb",
         "process_cells": "03_export2html.ipynb",
         "process_cell": "03_export2html.ipynb",
         "convert_nb": "03_export2html.ipynb",
         "notebook2html": "03_export2html.ipynb",
         "convert_md": "03_export2html.ipynb",
         "nb_detach_cells": "03_export2html.ipynb",
         "create_default_sidebar": "03_export2html.ipynb",
         "make_sidebar": "03_export2html.ipynb",
         "get_all_flags": "04_test.ipynb",
         "get_cell_flags": "04_test.ipynb",
         "NoExportPreprocessor": "04_test.ipynb",
         "test_nb": "04_test.ipynb",
         "extract_cells": "05_merge.ipynb",
         "get_md_cell": "05_merge.ipynb",
         "conflicts": "05_merge.ipynb",
         "same_inputs": "05_merge.ipynb",
         "analyze_cell": "05_merge.ipynb",
         "fix_conflicts": "05_merge.ipynb",
         "write_conda_meta": "05a_conda.ipynb",
         "CellMigrator": "06_cli.ipynb",
         "nbdev_upgrade": "06_cli.ipynb",
         "nbdev_build_lib": "06_cli.ipynb",
         "nbdev_update_lib": "06_cli.ipynb",
         "nbdev_diff_nbs": "06_cli.ipynb",
         "nbdev_test_nbs": "06_cli.ipynb",
         "make_readme": "06_cli.ipynb",
         "nbdev_build_docs": "06_cli.ipynb",
         "nbdev_nb2md": "06_cli.ipynb",
         "nbdev_detach": "06_cli.ipynb",
         "nbdev_read_nbs": "06_cli.ipynb",
         "nbdev_trust_nbs": "06_cli.ipynb",
         "nbdev_fix_merge": "06_cli.ipynb",
         "bump_version": "06_cli.ipynb",
         "nbdev_bump_version": "06_cli.ipynb",
         "nbdev_conda_package": "06_cli.ipynb",
         "nbdev_install_git_hooks": "06_cli.ipynb",
         "nbdev_new": "06_cli.ipynb",
         "rm_execution_count": "07_clean.ipynb",
         "clean_output_data_vnd": "07_clean.ipynb",
         "colab_json": "07_clean.ipynb",
         "clean_cell_output": "07_clean.ipynb",
         "cell_metadata_keep": "07_clean.ipynb",
         "nb_metadata_keep": "07_clean.ipynb",
         "clean_cell": "07_clean.ipynb",
         "clean_nb": "07_clean.ipynb",
         "nbdev_clean_nbs": "07_clean.ipynb",
         "say_hello": "tutorial.ipynb",
         "HelloSayer": "tutorial.ipynb"}

modules = ["export.py",
           "sync.py",
           "showdoc.py",
           "export2html.py",
           "test.py",
           "merge.py",
           "conda.py",
           "cli.py",
           "clean.py",
           "tutorial.py"]

doc_url = "https://nbdev.fast.ai/"

git_url = "https://github.com/fastai/nbdev/tree/master/"

def custom_doc_links(name): return None
